#!/bin/bash

# (C) 2006 Oden Eriksson <oeriksson@mandriva.com>
# Modified by Luigi Iotti <asterisk@iotti.biz>
# This program is Free Software under the GNU General Public License (>=v2).
# Read the file COPYING that comes with this packages for details.

if [ "x`whoami`" != "xroot" ]; then
    echo 'You must be root to run this.'
    exit
fi

echo ""
echo "[ freepbx_bootstrap ] Lux Configuration Script for FreePBX"
echo ""
echo "   * This script will configure FreePBX"
echo ""
echo "   * You _must_ read INSTALL completely first.  "
echo "     Some Manual steps are still required."
echo ""
echo "UNSUPPORTED -  Please use at your own risk.   "
echo ""
echo -n "Hit Return to continue or Ctrl-C to exit : "
read foo

DATE=`date +%Y%m%d%N`
if [ ! -d /etc/asterisk ]; then
    echo -e "\n/etc/asterisk either does not exist or is not a directory - exiting" >&2
    echo "You can create it by executing:" >&2
    echo " - mkdir /etc/asterisk" >&2
    echo " - chmod 775 /etc/asterisk" >&2
    echo -e " - chown -R asterisk:asterisk /etc/asterisk\n" >&2
    exit 1
fi
if [ ! -f /etc/freepbx.conf ]; then
    echo -e "\n/etc/freepbx.conf does not exist, but is needed to continue - exiting" >&2
    echo -e "You can reinstall the freepbx package, or take the file from some sample directory\n" >&2
    exit 1
fi
if [ ! -f /etc/freepbx/amportal.conf ]; then
    echo -e "\n/etc/freepbx/amportal.conf does not exist, but is needed to continue - exiting" >&2
    echo -e "You can reinstall the freepbx package, or take the file from some sample directory\n" >&2
    exit 1
fi
if [ ! -f /etc/asterisk/cdr_mysql.conf ]; then
    echo -e "\n/etc/asterisk/cdr_mysql.conf does not exist, but is needed to continue - exiting" >&2
    echo -e "It's likely you can take the file from some sample directory\n" >&2
    exit 1
fi
if [ ! -f /etc/asterisk/manager.conf ]; then
    echo -e "\n/etc/asterisk/manager.conf does not exist, but is needed to continue - exiting" >&2
    echo -e "It's likely you can take the file from some sample directory\n" >&2
    exit 1
fi

# Now these files are put in place by the rpm.

#echo "Saving old Asterisk configuration..."
#pushd /usr/share/freepbx/asterisk >/dev/null 2>&1
#for f in *.conf; do
#    if [ -f /etc/asterisk/${f} ]; then
#	echo "Saving old /etc/asterisk/${f} as /etc/asterisk/${f}-freepbx_backup-${DATE}"
#	mv /etc/asterisk/${f} /etc/asterisk/${f}-freepbx_backup-${DATE}
#    fi
#    cp ${f} /etc/asterisk/
#    chmod 664 /etc/asterisk/${f}
#    chown asterisk:asterisk /etc/asterisk/${f}
#done
#cp *.sample *.template *.inc /etc/asterisk/
#chmod 664 /etc/asterisk/*.sample /etc/asterisk/*.template /etc/asterisk/*.inc
#chown asterisk:asterisk /etc/asterisk/*.sample /etc/asterisk/*.template /etc/asterisk/*.inc
#popd >/dev/null 2>&1

# AMPDBUSER: the user to connect to the database named 'asterisk'
AMPDBUSER=`grep '^AMPDBUSER' /etc/freepbx/amportal.conf|cut -d= -f2`
# AMPDBPASS: the password for AMPDBUSER
AMPDBPASS=`perl -e 'for ($i = 0, $bit = "!", $key = ""; $i < 8; $i++) {while ($bit !~ /^[0-9A-Za-z]$/) { $bit = chr(rand(90) + 32); } $key .= $bit; $bit = "!"; } print "$key";'`
echo -n "Please enter the FreePBX Database password for the AMPDBUSER ($AMPDBUSER) user, press enter for the generated one (${AMPDBPASS}): "
read AMPDBPASS_db_passwd
if [ -z ${AMPDBPASS_db_passwd} ]; then AMPDBPASS_db_passwd=${AMPDBPASS}; fi
echo "Saving old /etc/freepbx/amportal.conf as /etc/freepbx/amportal.conf-backup-${DATE}"
cp -p /etc/freepbx/amportal.conf /etc/freepbx/amportal.conf-backup-${DATE}
echo "Saving old /etc/asterisk/cdr_mysql.conf as /etc/asterisk/cdr_mysql.conf-backup-${DATE}"
cp -p /etc/asterisk/cdr_mysql.conf /etc/asterisk/cdr_mysql.conf-backup-${DATE}
#sed -r -i "s/^AMPDBPASS=[a-zA-Z0-9_]*/AMPDBPASS=${AMPDBPASS_db_passwd}/" /etc/freepbx/amportal.conf
# In newer amportal.conf, AMPDBPASS is commented by default
sed -r -i "s/^[# ]*AMPDBPASS=[a-zA-Z0-9_]*/AMPDBPASS=${AMPDBPASS_db_passwd}/" /etc/freepbx/amportal.conf
sed -r -i "s/%AMPDBPASS%/${AMPDBPASS_db_passwd}/" /etc/freepbx.conf
sed -r -i "s/password *= *[a-zA-Z0-9_]*/password=${AMPDBPASS_db_passwd}/" /etc/asterisk/cdr_mysql.conf
sed -r -i "s/user *= *[a-zA-Z0-9_]*/user=$AMPDBUSER/" /etc/asterisk/cdr_mysql.conf

# AMPDBHOST: the host to connect to the database named 'asterisk'
echo -n "Please enter the FreePBX Database host, press enter for the default hostname (localhost): "
read AMPDBHOST_hostname
if [ -z ${AMPDBHOST_hostname} ]; then AMPDBHOST_hostname=localhost; fi
sed -r -i "s/hostname=[a-zA-Z0-9_]*/hostname=${AMPDBHOST_hostname}/" /etc/asterisk/cdr_mysql.conf
sed -r -i "s/%AMPDBHOST%/${AMPDBHOST_hostname}/" /etc/freepbx.conf

grep -q 'AMPDBNAME=' /etc/freepbx/amportal.conf || echo -e "\n# Lux\nAMPDBNAME=asterisk" >> /etc/freepbx/amportal.conf
grep -q 'ASTETCDIR=' /etc/freepbx/amportal.conf || echo -e "\n# Lux\ASTETCDIR=/etc/asterisk" >> /etc/freepbx/amportal.conf

# AMPMGRUSER: the user to access the Asterisk manager interface
AMPMGRUSER=`grep '^AMPMGRUSER' /etc/freepbx/amportal.conf|cut -d= -f2`
# AMPMGRPASS: the password for AMPMGRUSER
AMPMGRPASS=`perl -e 'for ($i = 0, $bit = "!", $key = ""; $i < 8; $i++) {while ($bit !~ /^[0-9A-Za-z]$/) { $bit = chr(rand(90) + 32); } $key .= $bit; $bit = "!"; } print "$key";'`
echo -n "Please enter the FreePBX Database password for the AMPMGRUSER ($AMPMGRUSER) user, press enter for the generated one (${AMPMGRPASS}): "
read AMPMGRPASS_db_passwd
if [ -z ${AMPMGRPASS_db_passwd} ]; then AMPMGRPASS_db_passwd=${AMPMGRPASS}; fi
#sed -r -i "s/^AMPMGRPASS=[a-zA-Z0-9_]*/AMPMGRPASS=${AMPMGRPASS_db_passwd}/" /etc/freepbx/amportal.conf
# In newer amportal.conf, AMPMGRPASS is commented by default
sed -r -i "s/^[# ]*AMPMGRPASS=[a-zA-Z0-9_]*/AMPMGRPASS=${AMPMGRPASS_db_passwd}/" /etc/freepbx/amportal.conf
sed -r -i "s/secret = [a-zA-Z0-9_]*/secret = ${AMPMGRPASS_db_passwd}/" /etc/asterisk/manager.conf
sed -r -i "/\[general\]/!s/\[[a-zA-Z0-9]+\]/[$AMPMGRUSER]/" /etc/asterisk/manager.conf
sed -r -i "s/^ARI_ADMIN_PASSWORD=[a-zA-Z0-9_]*/ARI_ADMIN_PASSWORD=${AMPMGRPASS_db_passwd}/" /etc/freepbx/amportal.conf

# FOPPASSWORD: the secret code for performing transfers and hangups in the Flash Operator Panel
FOPPASSWORD=`perl -e 'for ($i = 0, $bit = "!", $key = ""; $i < 8; $i++) {while ($bit !~ /^[0-9A-Za-z]$/) { $bit = chr(rand(90) + 32); } $key .= $bit; $bit = "!"; } print "$key";'`
echo -n "Please enter the FreePBX FOPPASSWORD, press enter for the generated one (${FOPPASSWORD}): "
read FOPPASSWORD_passwd
if [ -z ${FOPPASSWORD_passwd} ]; then FOPPASSWORD_passwd=${FOPPASSWORD}; fi
sed -r -i "s/^FOPPASSWORD=[a-zA-Z0-9_]*/FOPPASSWORD=${FOPPASSWORD_passwd}/" /etc/freepbx/amportal.conf

# AMPWEBADDRESS: the IP address or host name used to access the AMP web admin
AMPWEBADDRESS=`hostname --fqdn`
echo -n "Please enter the FreePBX web host, press enter for the detected hostname (${AMPWEBADDRESS}): "
read AMPWEBADDRESS_hostname
if [ -z ${AMPWEBADDRESS_hostname} ]; then AMPWEBADDRESS_hostname=${AMPWEBADDRESS}; fi
sed -r -i "s/^AMPWEBADDRESS=[a-zA-Z0-9]*/AMPWEBADDRESS=${AMPWEBADDRESS_hostname}/" /etc/freepbx/amportal.conf

if [ -e /var/www/freepbx/panel/op_server.cfg ]; then
    echo "Saving old /var/www/freepbx/panel/op_server.cfg as /var/www/freepbx/panel/op_server.cfg-backup-${DATE}"
    cp -p /var/www/freepbx/panel/op_server.cfg /var/www/freepbx/panel/op_server.cfg-backup-${DATE}
    FOPWEBROOT=`grep '^FOPWEBROOT' /etc/freepbx/amportal.conf|cut -d= -f2`
    sed -r -i "s/manager_user=[a-zA-Z0-9_]*/manager_user=$AMPMGRUSER/" /var/www/freepbx/panel/op_server.cfg
    sed -r -i "s/manager_secret=[a-zA-Z0-9_]*/manager_secret=${AMPMGRPASS_db_passwd}/" /var/www/freepbx/panel/op_server.cfg
    sed -r -i "s/web_hostname=[a-zA-Z0-9_.-]*/web_hostname=${AMPWEBADDRESS_hostname}/" /var/www/freepbx/panel/op_server.cfg
    sed -r -i "s/security_code=[a-zA-Z0-9_]*/security_code=${FOPPASSWORD_passwd}/" /var/www/freepbx/panel/op_server.cfg
    sed -r -i "s!flash_dir=[a-zA-Z0-9_./\-]*!flash_dir=$FOPWEBROOT!" /var/www/freepbx/panel/op_server.cfg
    sed -r -i "s!web_hostname=[a-zA-Z0-9.-]*!web_hostname=${AMPWEBADDRESS_hostname}!" /var/www/freepbx/panel/op_server.cfg
    sed -r -i "s!web_hostname=[a-zA-Z0-9.-]*!web_hostname=${AMPWEBADDRESS_hostname}!" /var/www/freepbx/panel/op_server.cfg
fi

if [ "xx${AMPWEBADDRESS_hostname}" = "xx" ]; then
	echo "You might need to modify /etc/asterisk/vm_email.inc manually"
else
	if [ -f /etc/asterisk/vm_email.inc ]; then
    		echo "Saving old /etc/asterisk/vm_email.inc as /etc/asterisk/vm_email.inc-backup-${DATE}"
    		cp -p /etc/asterisk/vm_email.inc /etc/asterisk/vm_email.inc-backup-${DATE}
		sed -r -i "s!http://.*/recordings!http://${AMPWEBADDRESS_hostname}/recordings!" /etc/asterisk/vm_email.inc
	fi
fi

# FOPPASSWORD: the secret code for performing transfers and hangups in the Flash Operator Panel
AMPWEBPASSWORD=`perl -e 'for ($i = 0, $bit = "!", $key = ""; $i < 8; $i++) {while ($bit !~ /^[0-9A-Za-z]$/) { $bit = chr(rand(90) + 32); } $key .= $bit; $bit = "!"; } print "$key";'`
echo -n "Please enter the password for the web admin user, press enter for the generated one (${AMPWEBPASSWORD}): "
read AMPWEBPASSWORD_passwd
if [ -z ${AMPWEBPASSWORD_passwd} ]; then AMPWEBPASSWORD_passwd=${AMPWEBPASSWORD}; fi
if [ -f /etc/freepbx/freepbx_web_users ]; then
	echo "Saving old /etc/freepbx/freepbx_web_users as /etc/freepbx/freepbx_web_users-backup-${DATE}"
	cp -p /etc/freepbx/freepbx_web_users /etc/freepbx/freepbx_web_users-backup-${DATE}
fi
echo ": clear text password for the admin user is: ${AMPWEBPASSWORD_passwd} :" >> /etc/freepbx/freepbx_web_users
# Now we use lighttpd
#htpasswd -m -b /etc/freepbx/freepbx_web_users admin ${AMPWEBPASSWORD_passwd}
lightdigest -u admin -p "${AMPWEBPASSWORD_passwd}" -r FreePBX -f /etc/freepbx/freepbx_web_users

echo "Now follow these steps to configure MySQL:

If this is the first time MySQL is used on this machine, then choose a password for MySQL' root user and do the following:

	- service mysqld start
	- chkconfig mysqld on
	- mysqladmin -u root password 'db_root_pwd'
	- mysql_secure_installation

###
Setting up MySQL for CDR and freePBX web interface

	- mysqladmin create asteriskcdrdb -p
	- mysql --user root -p asteriskcdrdb < /usr/share/doc/freepbx-2.10.0/cdr_mysql_table.sql
	
	- mysqladmin create asterisk -p
	- mysql --user root -p asterisk < /usr/share/doc/freepbx-2.10.0/newinstall.sql

###
Set the password for the web admin user
	- mysql --user root -p asterisk -e "UPDATE ampusers SET password = '${AMPWEBPASSWORD_passwd}' WHERE username = 'admin';"

	
###
Grant access to these two databases you just created

**Note the default mysql username/password is asteriskuser/amp109.
**If you change either of these, you will be prompted for them while running ./install below

	- mysql --user root -p

		mysql> GRANT ALL PRIVILEGES ON asteriskcdrdb.* TO ${AMPDBUSER}@localhost IDENTIFIED BY '${AMPDBPASS_db_passwd}';
		Query OK, 0 rows affected (0.00 sec)

		mysql> GRANT ALL PRIVILEGES ON asterisk.* TO ${AMPDBUSER}@localhost IDENTIFIED BY '${AMPDBPASS_db_passwd}';
		Query OK, 0 rows affected (0.00 sec)
		
		mysql> FLUSH PRIVILEGES;

		mysql> quit


###
(Re)-start Asterisk

	- service asterisk restart

###
Execute the freepbx_init script:

	- /usr/share/freepbx/freepbx_init --username=${AMPDBUSER} --password=${AMPDBPASS_db_passwd}

###
Change ownership of a log file just generated by install_amp, which should be writable by the asterisk user

	- chown asterisk:asterisk /var/log/asterisk/freepbx.log

###
(Re)-start freepbx
	- service freepbx restart

###
If using FOP, then start it
	- /usr/share/freepbx/module_admin reload
	- service op_server restart

###
Finally, activate the modules you need
	- /usr/share/freepbx/module_admin list
	- /usr/share/freepbx/module_admin install <module_name>
"


echo "Installation completed."
#echo "login via: http://${AMPWEBADDRESS_hostname}:81/ as admin and with the ${AMPWEBPASSWORD_passwd} password"
#echo "Try 'admin' password if ${AMPWEBPASSWORD_passwd} does not work (install_amp bug?). In case, change the password via the web interface"
echo "login via: http://${AMPWEBADDRESS_hostname}:81/ as admin and with admin password"
echo Remember to change the default password ASAP
