<?php
/*
 *Name         : functions.inc.php
 *Author       : Michael Yara
 *Created      : June 27, 2008
 *Last Updated : September 17, 2008
 *History      : 0.7
 *Purpose      : FreePBX module that automatically updates the iSymphony configuration from the FreePBX configuration.
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

//Build include path for iSymphony library
$inc_path = __FILE__;
$pos = strrpos($inc_path, "/");
$inc_path = substr($inc_path, 0, $pos) . "/";

//Include iSymphony library
require_once($inc_path . "/isymphony-php-library/isymphony.php");

//ParkAndAnnounce Class
class ext_isymphony_parkandannounce {
	var $template;
	var $timeout;
	var $dial;
	var $return_context;
	
	function ext_isymphony_parkandannounce($template, $timeout, $dial, $return_context) {
		$this->template = $template;
		$this->timeout = $timeout;
		$this->dial = $dial;
		$this->return_context = $return_context;
	}
	
	function output() {
		return "ParkAndAnnounce(" . $this->template . "|" . $this->timeout . "|" . $this->dial . "|" . $this->return_context . ")";
	}
}

//If check box GUI element does not exist add it
if(!class_exists("gui_checkbox")) {
	class gui_checkbox extends guiinput {
	       function gui_checkbox($elemname, $checked=false, $prompttext='', $helptext='', $value='on', $post_text = '', $jsonclick = '', $disable=false) {
	               $parent_class = get_parent_class($this);
	               parent::$parent_class($elemname, '', $prompttext, $helptext);
	
	               $itemchecked = $checked ? 'checked' : '';
	               $disable_state = $disable ? 'disabled="true"' : '';
	               $js_onclick_include = ($jsonclick != '') ? 'onclick="' . $jsonclick. '"' : '';
	               $tabindex = function_exists("guielement::gettabindex") ? "tabindex=" . guielement::gettabindex() : ""; 

	               $this->html_input = "<input type=\"checkbox\" name=\"$this->_elemname\" id=\"$this->_elemname\" $disable_state $tabindex value=\"$value\" $js_onclick_include $itemchecked/>$post_text\n";
	       }
	}
}


//Main contributing module function------------------------------------------------------------------------------------------------------------------
function isymphony_get_config($engine) {
	
	global $ext, $ISERROR, $amp_conf;
  	
  	switch($engine) {
  		case 'asterisk':	
  	
			ob_start();	
						
			$runningTimeStart = microtime(true);			
						
			//Open file for error log
			$errorLogFile = fopen("/var/www/html/admin/modules/isymphony/error.txt", 'w');
		
			//Open debug log
			if(($debugLogFile = fopen("/var/www/html/admin/modules/isymphony/debug.txt", 'w')) === false) {
				isymphony_write_to_file($errorLogFile, "Could not open debug log for writing.\n");
			}
						
			//Check if a manager profile exists for iSymphony if not create it.
			$managerFound = false;
			if((function_exists("manager_list")) && (($managers = manager_list()) !== null)) {
				
				//Search for iSymphony manager
				foreach($managers as $manager) {
					if($manager['name'] == "isymphony" ) {
						$managerFound = true;
						break;
					}
				}
			}

			//If not found create a manager profile for iSymphony
			if((function_exists("manager_add")) && (function_exists("manager_gen_conf")) && (!$managerFound)) {
				manager_add("isymphony", "ismanager*con", "0.0.0.0/0.0.0.0", "127.0.0.1/255.255.255.0", "system,call,log,verbose,command,agent,user", "system,call,log,verbose,command,agent,user");
				manager_gen_conf();
			}
		
			//Add custom iSymphony contexts
			$id = "musiconhold";
		    $c = '1000';
		    $ext->add($id, $c, '', new ext_answer(""));
		    $ext->add($id, $c, '', new ext_setvar("isHoldContext", "\${DB(iSymphonyChannelHoldContexts/\${CHANNEL})}"));
		    $ext->add($id, $c, '', new ext_musiconhold("\${isHoldContext}"));
		    $ext->add($id, $c, '', new ext_dbdel("iSymphonyChannelHoldContexts/\${CHANNEL}"));
		    
		    $id = "xfer-2-vm";
		    $c = "_.";
		    $ext->add($id, $c, '', new ext_goto("1", "\${VM_PREFIX}\${EXTEN}", "from-internal"));
		    $ext->add($id, $c, '', new ext_hangup());

		    $id = "ibarge";
			$ext->add($id, "_1X.", '', new ext_meetme("\${EXTEN:1}", "qd", "271721"));
		    $ext->add($id, "_1X.", '', new ext_hangup());
		    $ext->add($id, "_2X.", '', new ext_meetme("\${EXTEN:1}", "aq", "271721"));
		    $ext->add($id, "_2X.", '', new ext_hangup());

			$id = "imeetme";
			$c = "_.";
			$ext->add($id, "h", '', new ext_hangup());
			$ext->add($id, $c, '', new ext_setvar("roomOptions", "\${DB(iSymphonyMeetMeOptions/\${EXTEN})}"));
			$ext->add($id, $c, '', new ext_meetme("\${EXTEN}", "\${roomOptions}", "271721"));
			$ext->add($id, $c, '', new ext_hangup());

			$id = "ipark";
			$c = "_.";
			$ext->add($id, "h", '', new ext_hangup());
			$ext->add($id, $c, '', new ext_setvar("parkContext", "\${DB(iSymphonyParkContexts/\${EXTEN})}"));
			$ext->add($id, $c, '', new ext_answer(""));
			$ext->add($id, $c, '', new ext_wait("1"));
			$ext->add($id, $c, '', new ext_isymphony_parkandannounce("", "", "", "\${parkContext},\${EXTEN},1"));
			$ext->add($id, $c, '', new ext_dbdel("iSymphonyParkContexts/\${EXTEN}"));

			//Connect to iSymphony server to query and submit information
			if(iSymphonyConnect()) {
			
				//Check for device user mode
				$deviceUserMode = ($amp_conf["AMPEXTENSIONS"] == "deviceanduser") ? "true" : "false";
				
				//Check if default location exists if not create it else update manager connection values to stay consistent
				if(($iSymphonyLocations = getISymphonyLocationList()) !== false) {
					if(!in_array("default", $iSymphonyLocations)) {
						$object = new ISymphonyLocation;
						$object->name = "default";
						$object->admin_password = "secret";
						$object->asterisk_host = "localhost";
						$object->asterisk_port = "5038";
						$object->asterisk_login = "isymphony";
						$object->asterisk_password = "ismanager*con";
						$object->originate_timeout = "30000";
						$object->jabber_host = "";
						$object->jabber_port = "5222";
						$object->device_user_mode = $deviceUserMode;
						if($object->add()) {
							isymphony_write_to_file($debugLogFile, "Added location default\n");
						} else {
							isymphony_write_to_file($errorLogFile, "(Add location)" . $ISERROR. "\n");
						}
					} else {
						if(($object = getISymphonyLocation("default")) !== false) {
							
							//Check if update needs to occur
							if(($object->asterisk_host != "localhost") || ($object->asterisk_port != "5038") || ($object->asterisk_login != "isymphony") || ($object->asterisk_password != "ismanager*con") || ($object->device_user_mode != $deviceUserMode)) {		
									
								$object->asterisk_host = "localhost";
								$object->asterisk_port = "5038";
								$object->asterisk_login = "isymphony";
								$object->asterisk_password = "ismanager*con";
								$object->device_user_mode = $deviceUserMode;
								
								if($object->update()) {
									isymphony_write_to_file($debugLogFile, "Updated location default\n");
								} else {
									isymphony_write_to_file($errorLogFile, "(Update location)" . $ISERROR. "\n");	
								}
							}	
						} else {
							isymphony_write_to_file($errorLogFile, "(Query location)" . $ISERROR. "\n");
						}
					}
				} else {
					isymphony_write_to_file($errorLogFile, "(List locations)" . $ISERROR. "\n");
				}
	
				//Check if default tenant exists if not create it else update context values to stay consistent
				if(($iSymphonyTenants = getISymphonyTenantList("default")) !== false) {
					if(!in_array("default", $iSymphonyTenants)) {
						$object = new ISymphonyTenant;
						$object->name = "default";
						$object->admin_password = "secret";
						$object->originating_context = "from-internal";
						$object->redirecting_context = "from-internal";
						$object->music_on_hold_class = "default";
						$object->outside_line_number = "";
						$object->record_file_name = "%EXT%-%NAME%-%CID_NAME%-%CID_NUMBER%-%DATE%-%TIME%";
						$object->record_file_extension = "wav";
						$object->mix_mode = "true";
						if($object->add("default")) {
							isymphony_write_to_file($debugLogFile, "Added tenant default\n");
						} else {
							isymphony_write_to_file($errorLogFile, "(Add tenant)" . $ISERROR. "\n");
						}
					} else {
						if(($object = getISymphonyTenant("default", "default")) !== false) {
							
							//Check if update needs to occur
							if(($object->originating_context != "from-internal") || ($object->redirecting_context != "from-internal")) {
								
								$object->originating_context = "from-internal";
								$object->redirecting_context = "from-internal";
								
								if($object->update()) {
									isymphony_write_to_file($debugLogFile, "Updated tenant default\n");
								} else {
									isymphony_write_to_file($errorLogFile, "(Update tenant)" . $ISERROR. "\n");	
								}	
							}
						} else {
							isymphony_write_to_file($errorLogFile, "(Query tenant)" . $ISERROR. "\n");
						}
					}
				} else {
					isymphony_write_to_file($errorLogFile, "(List tenants)" . $ISERROR. "\n");
				}
	
				//Grab list of configured extensions from the database	
				$freePBXTempUsers = isymphony_user_list();
				
				//Filter list to exclude extensions that are not marked for addition
				$freePBXUsers = array();
				foreach($freePBXTempUsers as $freePBXTempUser) {
					if($freePBXTempUser["add_extension"] == "1") {
						array_push($freePBXUsers, $freePBXTempUser);
					}
				}
				
				//Filter the previous list for profiles excluding ones not marked for profile addition
				$freePBXProfiles = array();
				foreach($freePBXUsers as $freePBXUser) {
					if($freePBXUser["add_profile"] == "1") {
						array_push($freePBXProfiles, $freePBXUser);
					}
				}
	
				//Add, edit and remove extensions and profiles
				if((($iSymphonyExtensions = getISymphonyExtensionList("default", "default")) !== false) && (($iSymphonyProfiles = getISymphonyProfileList("default", "default")) !== false)) {
					
					//Build array of freePBX extensions to compare to iSymphony list for deletes 
					$freePBXExtensionCheckDeleteArray = array();
					foreach($freePBXUsers as $freePBXUser) {
						array_push($freePBXExtensionCheckDeleteArray, $freePBXUser["user_id"]);
					}
					
					//Delete appropriate extensions
					$deleteExtensionArray = array_diff($iSymphonyExtensions, $freePBXExtensionCheckDeleteArray);
					foreach($deleteExtensionArray as $val) {
						if(removeISymphonyExtension("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted extension {$val}\n");
							$iSymphonyExtensions = isymphony_remove_array_item($iSymphonyExtensions, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete extension)" . $ISERROR. "\n");
						}
					}
									
					//Add and update extensions
					foreach($freePBXUsers as $freePBXUser) {
						
						//If extension does not exist add it else update it
						if(!in_array($freePBXUser["user_id"], $iSymphonyExtensions)) {
							
							//Add extension
							$object = new ISymphonyExtension;
							$object->extension_val = $freePBXUser["user_id"];
							$object->name = $freePBXUser["display_name"];
							$object->peer = $freePBXUser["peer"];
							$object->alt_origination_method = "";
							$object->voice_mail = $freePBXUser["user_id"];
							
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added extension {$freePBXUser['user_id']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add extension)" . $ISERROR . "\n");
							}
						} else {
							
							//Query extension configuration and update values
							if(($object = getISymphonyExtension("default", "default", $freePBXUser["user_id"])) !== false) {		
													
								//Check if update needs to occur
								if(($object->name != $freePBXUser["display_name"]) || ($object->voice_mail != $freePBXUser["user_id"]) || ($object->peer != $freePBXUser["peer"])) {		
											
									$object->name = $freePBXUser["display_name"];
									$object->voice_mail = $freePBXUser["user_id"];
									$object->peer = $freePBXUser["peer"];
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated extension {$freePBXUser['user_id']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update extension)" . $ISERROR. "\n");	
									}	
								}
							} else {
								isymphony_write_to_file($errorLogFile, "(Query extension)" . $ISERROR. "\n"); 
							}
						}
					}
						
					//Build array of freePBX extensions to compare to iSymphony list for deletes 
					$freePBXProfileCheckDeleteArray = array();
					foreach($freePBXProfiles as $freePBXProfile) {
						array_push($freePBXProfileCheckDeleteArray, $freePBXProfile["user_id"]);
					}
						
					//Delete appropriate profiles
					$deleteProfileArray = array_diff($iSymphonyProfiles, $freePBXProfileCheckDeleteArray);
					foreach($deleteProfileArray as $val) {
						if(removeISymphonyProfile("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted profile {$val}\n");
							$iSymphonyProfiles = isymphony_remove_array_item($iSymphonyProfiles, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete profile)" . $ISERROR. "\n");
						}
					}	
						
					//Add and update profiles
					foreach($freePBXProfiles as $freePBXProfile) {
			
						//If profile does not exist add it else update it
						if(!in_array($freePBXProfile["user_id"], $iSymphonyProfiles)) {
							
							//Add profile
							$object = new ISymphonyProfile;
							$object->name = $freePBXProfile["user_id"];
							$object->password = $freePBXProfile["password"];
							$object->can_view_everyone_directory = "true";
							
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added profile {$freePBXProfile['user_id']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add profile)" . $ISERROR . "\n");
							}
						
							//Add extension to profiles managed list
							if(addISymphonyProfileManagedExtension("default","default",$freePBXProfile["user_id"],$freePBXProfile["user_id"])) {
								isymphony_write_to_file($debugLogFile, "Added profile managed extension {$freePBXProfile['user_id']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add profile managed extension)" . $ISERROR . "\n");
							}
						} else {
							
							//Query profile configuration and update values
							if(($object = getISymphonyProfile("default", "default", $freePBXProfile["user_id"])) !== false) {		
													
								//Check if update needs to occur
								if($object->password != $freePBXProfile["password"]) {		
											
									$object->password = $freePBXProfile["password"];
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated profile {$freePBXProfile['user_id']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update profile)" . $ISERROR. "\n");	
									}	
								}
							} else {
								isymphony_write_to_file($errorLogFile, "(Query profile)" . $ISERROR. "\n"); 
							}
						}												
					}
				} else {
					isymphony_write_to_file($errorLogFile, "(List extensions/profiles)" . $ISERROR. "\n");
				}	
				
				//Grab list of configured queues from the database	
				$freePBXTempQueues = isymphony_queue_list();
	
				//Filter list to exclude queues that are not marked for addition
				$freePBXQueues = array();
				foreach($freePBXTempQueues as $freePBXTempQueue) {
					if($freePBXTempQueue["add_queue"] == "1") {
						array_push($freePBXQueues, $freePBXTempQueue);
					}
				}
	
				//Add, edit and remove queues
				if(($iSymphonyQueues = getISymphonyQueueList("default", "default")) !== false) {
					
					//Build array of freePBX queues to compare to iSymphony list for deletes 
					$freePBXQueueCheckDeleteArray = array();
					foreach($freePBXQueues as $freePBXQueue) {
						array_push($freePBXQueueCheckDeleteArray, $freePBXQueue["display_name"]);
					}
				 	
				 	//Delete appropriate queues
					$deleteQueueArray = array_diff($iSymphonyQueues, $freePBXQueueCheckDeleteArray);
					foreach($deleteQueueArray as $val) {
						if(removeISymphonyQueue("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted queue {$val}\n");
							$iSymphonyQueues = isymphony_remove_array_item($iSymphonyQueues, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete queue)" . $ISERROR. "\n");
						}
					}
					
					//Add and update queues
				 	foreach($freePBXQueues as $freePBXQueue) {
				 					 		
				 		//If queue does not exist add it
						if(!in_array($freePBXQueue["display_name"], $iSymphonyQueues)) {
							
							//Add queue
							$object = new ISymphonyQueue;
							$object->name = $freePBXQueue["display_name"];
							$object->queue_val = $freePBXQueue["queue_id"];
							$object->extension_val = $freePBXQueue["queue_id"];
							$object->context = "from-internal";
	
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added queue {$freePBXQueue['display_name']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add queue)" . $ISERROR . "\n");
							}
							
						//If queue does exist update it		
						} else {
							
							//Query queue configuration and update values
							if(($object = getISymphonyQueue("default", "default", $freePBXQueue["display_name"])) !== false) {		
													
								//Check if update needs to occur
								if(($object->name != $freePBXQueue["display_name"]) || ($object->queue_val != $freePBXQueue["queue_id"]) || ($object->extension_val != $freePBXQueue["queue_id"]) || ($object->context != "from-internal")) {					
													
									$object->name = $freePBXQueue["display_name"];
									$object->queue_val = $freePBXQueue["queue_id"];
									$object->extension_val = $freePBXQueue["queue_id"];
									$object->context = "from-internal";
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated queue {$freePBXQueue['display_name']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update queue)" . $ISERROR. "\n");	
									}	
								}
							} else {
								isymphony_write_to_file($errorLogFile, "(Query queue)" . $ISERROR. "\n"); 
							}
						}	
				 	}				
				 } else {
				 	isymphony_write_to_file($errorLogFile, "(List queues)" . $ISERROR. "\n");
				 }
	
				//Grab list of configured conference rooms from the database	
				$freePBXTempConferenceRooms = isymphony_conference_room_list();
	
				//Filter list to exclude conference rooms that are not marked for addition
				$freePBXConferenceRooms = array();
				foreach($freePBXTempConferenceRooms as $freePBXTempConferenceRoom) {
					if($freePBXTempConferenceRoom["add_conference_room"] == "1") {
						array_push($freePBXConferenceRooms, $freePBXTempConferenceRoom);
					}
				}
	
				//Add, edit and remove conference rooms
				if(($iSymphonyConferenceRooms = getISymphonyConferenceRoomList("default", "default")) !== false) {
					
					//Build array of freePBX conference rooms to compare to iSymphony list for deletes 
					$freePBXConferenceRoomCheckDeleteArray = array();
					foreach($freePBXConferenceRooms as $freePBXConferenceRoom) {
						array_push($freePBXConferenceRoomCheckDeleteArray, $freePBXConferenceRoom["display_name"]);
					}
				 	
				 	//Delete appropriate conference rooms
					$deleteConferenceRoomArray = array_diff($iSymphonyConferenceRooms, $freePBXConferenceRoomCheckDeleteArray);
					foreach($deleteConferenceRoomArray as $val) {
						if(removeISymphonyConferenceRoom("default", "default", $val)) {
							isymphony_write_to_file($debugLogFile, "Deleted conference room {$val}\n");
							$iSymphonyConferenceRooms = isymphony_remove_array_item($iSymphonyConferenceRooms, $val);
						} else {
							isymphony_write_to_file($errorLogFile, "(Delete conference room)" . $ISERROR. "\n");
						}
					}
					
					//Add and update conference rooms
				 	foreach($freePBXConferenceRooms as $freePBXConferenceRoom) {
				 					 		
				 		//If conference room does not exist add it
						if(!in_array($freePBXConferenceRoom["display_name"], $iSymphonyConferenceRooms)) {
							
							//Add conference room
							$object = new ISymphonyConferenceRoom;
							$object->name = $freePBXConferenceRoom["display_name"];
							$object->predefined = "true";
							$object->room_number = $freePBXConferenceRoom["conference_room_id"];
							$object->extension_val = $freePBXConferenceRoom["conference_room_id"];
							$object->context = "from-internal";
							$object->announce_user_count = "false";
							$object->music_on_hold_for_single_user = "false";
							$object->exit_room_via_pound = "false";
							$object->present_menu_via_star = "false";
							$object->announce_user_join_leave = "false";
							$object->disable_join_leave_notification = "false";
							$object->record = "false";
	
							if($object->add("default","default")) {
								isymphony_write_to_file($debugLogFile, "Added conference room {$freePBXConferenceRoom['display_name']}\n");
							} else {
								isymphony_write_to_file($errorLogFile, "(Add conference room)" . $ISERROR . "\n");
							}
							
						//If conference room does exist update it		
						} else {
							
							//Query conference room configuration and update values
							if(($object = getISymphonyConferenceRoom("default", "default", $freePBXConferenceRoom["display_name"])) !== false) {		
												
								//Check if update needs to occur
								if(($object->name != $freePBXConferenceRoom["display_name"]) || ($object->predefined != "true") || ($object->room_number != $freePBXConferenceRoom["conference_room_id"]) || ($object->extension_val != $freePBXConferenceRoom["conference_room_id"]) || ($object->context != "from-internal")) {
										
									$object->name = $freePBXConferenceRoom["display_name"];
									$object->predefined = "true";
									$object->room_number = $freePBXConferenceRoom["conference_room_id"];
									$object->extension_val = $freePBXConferenceRoom["conference_room_id"];
									$object->context = "from-internal";
									
									if($object->update()) {
										isymphony_write_to_file($debugLogFile, "Updated conference room {$freePBXConferenceRoom['display_name']}\n");
									} else {
										isymphony_write_to_file($errorLogFile, "(Update conference room)" . $ISERROR. "\n");	
									}
								}	
							} else {
								isymphony_write_to_file($errorLogFile, "(Query conference room)" . $ISERROR. "\n"); 
							}
						}	
				 	}				
				} else {
					isymphony_write_to_file($errorLogFile, "(List conference rooms)" . $ISERROR. "\n");
				}	
	
				//Close iSymphony connection
				iSymphonyDisconnect();
			
  			} else {
  				isymphony_write_to_file($errorLogFile, $ISERROR. "\n");
  			} 
			
			$runningTimeStop = microtime(true);
			isymphony_write_to_file($debugLogFile, "Total Running Time:" . ($runningTimeStop - $runningTimeStart) . "s\n");
			
			//Close file handlers
			fclose($debugLogFile);
			fclose($errorLogFile);
		
			ob_end_clean();
			break;
  	}
}

//iSymphony module API user functions----------------------------------------------------------------------------------------------------------------
function isymphony_user_add($userId, $addExtension, $addProfile, $password, $displayName, $peer) {
	global $db;	
	$addProfile = $addProfile ? "1" : "0";
	$addExtension = $addExtension ? "1" : "0";
	$prepStatement = $db->prepare("INSERT INTO isymphony_users (user_id, add_extension, add_profile, password, display_name, peer) VALUES (?, ?, ?, ?, ?, ?)");
	$values = array($userId, $addExtension, $addProfile, $password, $displayName, $peer);
	$db->execute($prepStatement, $values);
}

function isymphony_user_update($userId, $addExtension, $addProfile, $password, $displayName, $peer) {
	global $db;	
	$addProfile = $addProfile ? "1" : "0";
	$addExtension = $addExtension ? "1" : "0";
	$prepStatement = $db->prepare("UPDATE isymphony_users SET add_extension = ?, add_profile = ?, password = ?, display_name = ?, peer = ? WHERE user_id = $userId");
	$values = array($addExtension, $addProfile, $password, $displayName, $peer);
	$db->execute($prepStatement, $values);
}

function isymphony_user_del($userId) {
	global $db;	
	$query = "DELETE FROM isymphony_users WHERE user_id = '$userId'";
    $db->query($query);
}

function isymphony_user_list() {
	global $db;	
	$query = "SELECT * FROM isymphony_users";
	$results = sql($query, "getAll", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return array();
	} else {
		return $results;
	}
}

function isymphony_user_get($userId) {
	global $db;	
	$query = "SELECT * FROM isymphony_users WHERE user_id = '$userId'";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//iSymphony module API queue functions---------------------------------------------------------------------------------------------------------------
function isymphony_queue_add($queueId, $addQueue, $displayName) {
	global $db;
	$addQueue = $addQueue ? "1" : "0";	
	$prepStatement = $db->prepare("INSERT INTO isymphony_queues (queue_id, add_queue, display_name) VALUES (?, ?, ?)");
	$values = array($queueId, $addQueue, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_queue_update($queueId, $addQueue, $displayName) {
	global $db;	
	$addQueue = $addQueue ? "1" : "0";	
	$prepStatement = $db->prepare("UPDATE isymphony_queues SET add_queue = ?, display_name = ? WHERE queue_id = $queueId");
	$values = array($addQueue, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_queue_del($queueId) {
	global $db;	
	$query = "DELETE FROM isymphony_queues WHERE queue_id = '$queueId'";
    $db->query($query);
}

function isymphony_queue_list() {
	global $db;	
	$query = "SELECT * FROM isymphony_queues";
	$results = sql($query, "getAll", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return array();
	} else {
		return $results;
	}
}

function isymphony_queue_get($queueId) {
	global $db;	
	$query = "SELECT * FROM isymphony_queues WHERE queue_id = '$queueId'";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//iSymphony module API conference room functions-----------------------------------------------------------------------------------------------------
function isymphony_conference_room_add($conferenceRoomId, $addConferenceRoom, $displayName) {
	global $db;	
	$addConferenceRoom = $addConferenceRoom ? "1" : "0";	
	$prepStatement = $db->prepare("INSERT INTO isymphony_conference_rooms (conference_room_id, add_conference_room, display_name) VALUES (?, ?, ?)");
	$values = array($conferenceRoomId, $addConferenceRoom, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_conference_room_update($conferenceRoomId, $addConferenceRoom, $displayName) {
	global $db;	
	$addConferenceRoom = $addConferenceRoom ? "1" : "0";
	$prepStatement = $db->prepare("UPDATE isymphony_conference_rooms SET add_conference_room = ?, display_name = ? WHERE conference_room_id = $conferenceRoomId");
	$values = array($addConferenceRoom, $displayName);
	$db->execute($prepStatement, $values);
}

function isymphony_conference_room_del($conferenceRoomId) {
	global $db;	
	$query = "DELETE FROM isymphony_conference_rooms WHERE conference_room_id = '$conferenceRoomId'";
    $db->query($query);
}

function isymphony_conference_room_list() {
	global $db;	
	$query = "SELECT * FROM isymphony_conference_rooms";
	$results = sql($query, "getAll", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return array();
	} else {
		return $results;
	}
}

function isymphony_conference_room_get($conferenceRoomId) {
	global $db;	
	$query = "SELECT * FROM isymphony_conference_rooms WHERE conference_room_id = '$conferenceRoomId'";
	$results = sql($query, "getRow", DB_FETCHMODE_ASSOC);
	if((DB::IsError($results)) || (empty($results))) {
		return null;
	} else {
		return $results;
	}
}

//Extension/User page hooks--------------------------------------------------------------------------------------------------------------------------
function isymphony_configpageinit($pagename) {
	global $currentcomponent;

	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$extdisplay = isset($_REQUEST["extdisplay"]) ? $_REQUEST["extdisplay"] : null;
	$extension = isset($_REQUEST["extension"]) ? $_REQUEST["extension"] : null;
	$tech_hardware = isset($_REQUEST["tech_hardware"]) ? $_REQUEST["tech_hardware"] : null;

	//Based on the page state determine if the display or process functions should be added
	if (($pagename != "users") && ($pagename != "extensions")) { 
		return;
	} else if ($tech_hardware != null || $pagename == "users") {
		isymphony_extension_applyhooks();
		$currentcomponent->addprocessfunc('isymphony_extension_configprocess', 8);
	} elseif ($action == "add" || $action == "edit") {
		$currentcomponent->addprocessfunc('isymphony_extension_configprocess', 8);
	} elseif ($extdisplay != '') {
		isymphony_extension_applyhooks();
		$currentcomponent->addprocessfunc('isymphony_extension_configprocess', 8);
	}
}

function isymphony_extension_applyhooks() {
	global $currentcomponent;
	$currentcomponent->addguifunc("isymphony_extension_configpageload");
}

function isymphony_extension_configpageload() {
	global $currentcomponent;

	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$display = isset($_REQUEST["display"]) ? $_REQUEST["display"] : null;
	$extension = isset($_REQUEST["extdisplay"]) ? $_REQUEST["extdisplay"] : null;
	
	//Attempt to query element if not found set defaults
	if(($extension !== null) && (($iSymphonyUser = isymphony_user_get($extension)) !== null)) {
		$added = ($iSymphonyUser["add_extension"] == "1");
		$userProfile = ($iSymphonyUser["add_profile"] == "1");
		$password = $iSymphonyUser["password"];
	
	} else {
		$added = true;
		$userProfile = true;
		$password = "secret";	
	}
	
	//Create GIU elements if not on delete page
	if ($action != "del") {
		$section = _("iSymphony");
		$currentcomponent->addguielem($section,	new gui_checkbox("isymphony_add_extension", $added, "Add to iSymphony", "Makes this Extension/User available in iSymphony.", "on", "", "", false));
		$currentcomponent->addguielem($section,	new gui_checkbox("isymphony_add_profile", $userProfile, "Create Profile", "Creates an iSymphony login profile which is associated with this extension.", "on", "", "", false));
		$currentcomponent->addguielem($section, new gui_textbox("isymphony_profile_password", $password, "Profile Password", "Specifies the password to be used for this profile.", "", "", true, "100", false));	
	}
}

function isymphony_extension_configprocess() {
	
	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$ext = isset($_REQUEST["extdisplay"]) ? $_REQUEST["extdisplay"] : null;
	$extn = isset($_REQUEST["extension"]) ? $_REQUEST["extension"]: null;
	$name = isset($_REQUEST["name"]) ? $_REQUEST["name"] : null;
	$extension = ($ext === "") ? $extn : $ext;
	
	//Determine peer
	if(isset($_REQUEST["devinfo_dial"]) && ($_REQUEST["devinfo_dial"] != "")) {
		$peer = $_REQUEST["devinfo_dial"];
	} else if (isset($_REQUEST["tech"])){
		$peer = strtoupper($_REQUEST["tech"]) . "/" . $extension;
	} else {
		$peer = "SIP/$extension";
	}
		
	//Query iSymphony options
	$addExtension = isset($_REQUEST["isymphony_add_extension"]);
	$addProfile = isset($_REQUEST["isymphony_add_profile"]);
	$password = isset($_REQUEST["isymphony_profile_password"]) ? $_REQUEST["isymphony_profile_password"]: null;
	$password = (($password === null) || (trim($password) == "")) ? "secret" : trim($password);
		
	//Modify DB
	if(($extension !== null) && ($extension != "") && ($action !== null)) {
		
		//Check if this extension needs to be deleted, updated, or added
		if($action == "del") {
			isymphony_user_del($extension);
		} else if(($action == "add") || ($action == "edit") && ($name !== null)) {
			if(isymphony_user_get($extension) === null) {
				isymphony_user_add($extension, $addExtension, $addProfile, $password, $name, $peer);
			} else {
				isymphony_user_update($extension, $addExtension, $addProfile, $password, $name, $peer);
			}
		}
	}
}

//Queue page hooks-----------------------------------------------------------------------------------------------------------------------------------
function isymphony_hook_queues($viewing_itemid, $target_menuid) {
		
	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$display = "";
	
	//Only hook queues page
  	if(($target_menuid == "queues") && ($action != "delete")) {
  		
  		//Query queue info
  		if(($viewing_itemid != null) && ($iSymphonyQueue = isymphony_queue_get($viewing_itemid))) {
  			$checked = ($iSymphonyQueue["add_queue"] == "1") ? "checked" : "";
  		} else {
  			$checked = "checked";
  		}
  		
  		//Build display
  		$display = "	<tr><td colspan=\"2\"><h5>iSymphony<hr></h5></td></tr>
						<tr>
							<td><a href=\"#\" class=\"info\">" . _("Add to iSymphony") . "<span>" . _("Makes this queue available in iSymphony") . "</span></a></td>
							<td><input type=\"checkbox\" name=\"isymphony_add_queue\" id=\"isymphony_add_queue\" value=\"on\" $checked/></td>
						</tr>";
  	}
 
  	return $display;
}

function isymphony_hookProcess_queues($viewing_itemid, $request) {

	//Query page state
	$queue = isset($request["extdisplay"]) ? $request["extdisplay"] : null;
	$account = isset($request["account"]) ? $request["account"] : null;
	$action = isset($request["action"]) ? $request["action"] : null;
	$name = isset($request["name"]) ? $request["name"] : null;
	$queue = ($queue == null) ? $account : $queue;
	
	//Query iSymphony option
	$addQueue = isset($request["isymphony_add_queue"]);
	
	//Update DB
	if(($queue != null) && ($queue != "") && ($action != null)) {
		
		//Check if this queue needs to be deleted, updated, or added
		if($action == "delete") {
			isymphony_queue_del($queue);
		} else if(($action == "add") || ($action == "edit") && ($name !== null)) {
			if(isymphony_queue_get($queue) === null) {
				isymphony_queue_add($queue, $addQueue, $name);
			} else {
				isymphony_queue_update($queue, $addQueue, $name);
			}
		}
	}
}

//Conference Room page hooks-------------------------------------------------------------------------------------------------------------------------
function isymphony_hook_conferences($viewing_itemid, $target_menuid) {
		
	//Query page state
	$action = isset($_REQUEST["action"]) ? $_REQUEST["action"] : null;
	$display = "";
	
	//Only hook conferences page
  	if(($target_menuid == "conferences") && ($action != "delete")) {
  		
  		//Query conference info
  		if(($viewing_itemid != null) && ($iSymphonyConferenceRoom = isymphony_conference_room_get($viewing_itemid))) {
  			$checked = ($iSymphonyConferenceRoom["add_conference_room"] == "1") ? "checked" : "";
  		} else {
  			$checked = "checked";
  		}
  		
  		//Build display
  		$display = "	<tr><td colspan=\"2\"><h5>iSymphony<hr></h5></td></tr>
						<tr>
							<td><a href=\"#\" class=\"info\">" . _("Add to iSymphony") . "<span>" . _("Makes this conference room available in iSymphony") . "</span></a></td>
							<td><input type=\"checkbox\" name=\"isymphony_add_conference_room\" id=\"isymphony_add_conference_room\" value=\"on\" $checked/></td>
						</tr>";
  	}
 
  	return $display;
}

function isymphony_hookProcess_conferences($viewing_itemid, $request) {

	//Query page state
	$conferenceRoom = isset($request["extdisplay"]) ? $request["extdisplay"] : null;
	$account = isset($request["account"]) ? $request["account"] : null;
	$action = isset($request["action"]) ? $request["action"] : null;
	$name = isset($request["name"]) ? $request["name"] : null;
	$conferenceRoom = ($conferenceRoom == null) ? $account : $conferenceRoom;
	
	//Query iSymphony option
	$addConferenceRoom = isset($request["isymphony_add_conference_room"]);
	
	//Update DB
	if(($conferenceRoom != null) && ($conferenceRoom != "") && ($action != null)) {
		
		//Check if this conference room needs to be deleted, updated, or added
		if($action == "delete") {
			isymphony_conference_room_del($conferenceRoom);
		} else if(($action == "add") || ($action == "edit") && ($name !== null)) {
			if(isymphony_conference_room_get($conferenceRoom) === null) {
				isymphony_conference_room_add($conferenceRoom, $addConferenceRoom, $name);
			} else {
				isymphony_conference_room_update($conferenceRoom, $addConferenceRoom, $name);
			}
		}
	}
}

//Helper functions-----------------------------------------------------------------------------------------------------------------------------------
function isymphony_write_to_file($file, $content) {
	if($file) {
		fwrite($file,$content);
	}
}

function isymphony_remove_array_item($array, $item) {
	return explode(',',str_replace($item.',','',(join(',',$array))));
}
?>