<?php
/*
 *Name         : classes.php
 *Author       : Michael Yara
 *Created      : June 15, 2008
 *Last Updated : August 22, 2008
 *History      : 0.4
 *Purpose      : Contains declarations of all property classes. 
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

/*###############################################
 * Class that represents the server properties
 * Attributes: 
 *		client_port: Port that server listens for client connections on.(INTEGER)
 *		cli_port: Port that servers listens for CLI requests on.(INTEGER)
 *		username: Admin username.(STRING)
 *		password: Admin password.(STRING)
 *
 * Methods:
 *		update: Commits changes made to the property configuration
 *			Takes: nothing
 *			Returns: true if successful false if not
 */
 class ISymphonyServer {
 	
 	//Property attributes
 	var $client_port = "";
 	var $cli_port = "";
 	var $username = "";
 	var $password = "";
 	 		
 	//Methods
 	function update() {
 		
 		global $serverPropertyArray;
 		
 		//Move to server mode
		if(!moveToServer()) {
			return false;
		}
		
		//Modify properties
		foreach($serverPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		return checkAndSetErrorNone("commit");
 	}
 }

/*###############################################
 * Class that represents a locations's properties
 * Attributes: 
 *		name: Location name(STRING)
 *		admin_password: Admin password(STRING)
 *		asterisk_host: Asterisk manager host name or IP(STRING)
 *		asterisk_port: Asterisk manager port(INTEGER)
 *		asterisk_login: Asterisk manager username(STRING)
 *		asterisk_password: Asterisk manager password(STRING)
 *		originate_timeout: Amount of time an originator will be rung on an origination event in milliseconds(INTEGER)
 *		jabber_host: Jabber host name(STRING)
 *		jabber_port: Jabber port(INTEGER)
 *		device_user_mode: (BOOLEAN[true or false])
 *
 * Methods:
 *		update: Commits changes made to the property configuration
 *			Takes: nothing
 *			Returns: true if successful false if not
 *		add: Adds the given location with specified configuration
 *			Takes: nothing
 *			Returns: true if successful false if not
 */
 class ISymphonyLocation {
 	
 	//Mode attributes
 	var $original_name = "";
 	
 	//Property attributes
	var $name = "";
	var $admin_password = "";
	var $asterisk_host = "";
	var $asterisk_port = "";
	var $asterisk_login = "";
	var $asterisk_password = "";
	var $originate_timeout = "";
	var $jabber_host = "";
	var $jabber_port = "";
	var $device_user_mode = "";
	
 	//Methods
 	function update() {
 		
 		global $locationPropertyArray;
 		
 		//Move to location mode
		if(!moveToLocation($this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($locationPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add() {
 		
 		global $locationPropertyArray;
 		
 		//Move to server mode
		if(!moveToServer()) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new location {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($locationPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 }

/*###############################################
 * Class that represents a tenants's properties
 * Attributes: 
 *		name: Name of the tenant(STRING)
 *		admin_password: Admin password(STRING)
 *		originating_context: Context used for originating calls(STRING)
 *		redirecting_context: Context used for redirecting calls(STRING)
 *		music_on_hold_class: Music on hold class(STRING)
 *		outside_line_number: Number used to dial outside numbers(STRING)
 *		record_file_name: File name used for recorded calls. See code legend in administration interface for a list of input variables(STRING)
 *		record_file_extension: File extensions used for recorded calls(STRING)
 *		mix_mode: Flag used to mix incoming and outgoing recorded call channels(BOOLEAN[true or false])
 *
 * Methods:
 *		update: Commits changes made to the property configuration
 *			Takes: nothing
 *			Returns: true if successful false if not
 *		add: Adds the given tenant with specified configuration
 *			Takes: the location you wish to add the tenant to
 *			Returns: true if successful false if not
 */
 class ISymphonyTenant {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $original_name = "";
 	
 	//Property attributes
	var $name = "";
	var $admin_password = "";
	var $originating_context = "";
	var $redirecting_context = "";
	var $music_on_hold_class = "";
	var $outside_line_number = "";
	var $record_file_name = "";
	var $record_file_extension = "";
	var $mix_mode = "";
	
 	//Methods
 	function update() {
 		
 		global $tenantPropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($this->mode_location, $this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($tenantPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location) {
 		
 		global $tenantPropertyArray;
 		
 		//Move to location mode
		if(!moveToLocation($location)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new tenant {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($tenantPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			$this->mode_location = $location;
			return true;
			
		} else {
			return false;
		}
 	}
 }
 
 /*###############################################
  * Class that represents an extension's properties
  * Attributes: 
  *		extension_val: Extension number(STRING)
  *		name: Extension display name(STRING)
  *		cell_phone: Cell phone number(STRING)
  *		email: Email address(STRING)
  *		peer: Peer associated with this extension(STRING)
  *		alt_origination_method: Alternate method for originating calls if it differers from the peer(STRING)
  *		voice_mail: Voice mail box number of this extension(STRING)
  *		agent: Agent number associated with this extension(STRING)
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given tenant with specified configuration
  *			Takes: the location you wish to add the extension to, the tenant that you would like to add the extension to
  *			Returns: true if successful false if not
  */
  class ISymphonyExtension {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $original_extension_val = "";
 	
 	//Property attributes
	var $extension_val = "";
	var $name = "";
	var $cell_phone = "";
	var $email = "";
	var $peer = "";
	var $alt_origination_method = "";
	var $voice_mail = "";
	var $agent = "";
	
 	//Methods
 	function update() {
 		
 		global $extensionPropertyArray;
 		
 		//Move to extension mode
		if(!moveToExtension($this->mode_location, $this->mode_tenant, $this->original_extension_val)) {
			return false;
		}
		
		//Modify properties
		foreach($extensionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_extension_val = $this->extension_val;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant) {
 		
 		global $extensionPropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new extension {$this->extension_val}")) {
			return false;
		}
		
		//Modify properties
		foreach($extensionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_extension_val = $this->extension_val;
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			return true;
			
		} else {
			return false;
		}
 	}
 }
 
  /*###############################################
  * Class that represents a profile's properties
  * Attributes: 
  * 	name: Username for profile(STRING)
  * 	password: Password for profile(STRING)
  * 	jabber_user_name: Jabber username(STRING)
  * 	jabber_password: Jabber password(STRING)
  * 	can_view_everyone_directory: Flag that allows this profile to see the Everyone directory(BOOLEAN[true or false])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given profile with specified configuration
  *			Takes: the location you wish to add the profile to, the tenant that you would like to add the profile to
  *			Returns: true if successful false if not
  */
  class ISymphonyProfile {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $original_name = "";
 	
 	//Property attributes
	var $name = "";
	var $password = "";
	var $jabber_user_name = "";
	var $jabber_password = "";
	var $can_view_everyone_directory = "";
	
 	//Methods
 	function update() {
 		
 		global $profilePropertyArray;
 		
 		//Move to profile mode
		if(!moveToProfile($this->mode_location, $this->mode_tenant, $this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($profilePropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant) {
 		
 		global $profilePropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new profile {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($profilePropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			return true;
			
		} else {
			return false;
		}
 	}
 }
 
 /*###############################################
  * Class that represents a queue's properties
  * Attributes: 
  *		name: Display name of queue(STRING)
  *		queue_val: Name of queue as defined in queues.conf(STRING)
  *		extension_val: Extension used to access queue(STRING)
  *		context: Context used to access queue(STRING)
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given queue with specified configuration
  *			Takes: the location you wish to add the queue to, the tenant that you would like to add the queue to
  *			Returns: true if successful false if not
  */
  class ISymphonyQueue {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $original_name = "";
 	
 	//Property attributes
	var $name = "";
	var $queue_val = "";
	var $extension_val = "";
	var $context = "";
	
 	//Methods
 	function update() {
 		
 		global $queuePropertyArray;
 		
 		//Move to queue mode
		if(!moveToQueue($this->mode_location, $this->mode_tenant, $this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($queuePropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant) {
 		
 		global $queuePropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new queue {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($queuePropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			return true;
			
		} else {
			return false;
		}
 	}
 }
 
 /*###############################################
  * Class that represents a conference room's properties
  * Attributes: 
  *		name: Display name of conference room(STRING)
  *		predefined: Flag describing if this is a predefined or custom room(BOOLEAN[true or false])
  *		room_number: Room number for predefined room as it appears in meetme.conf(STRING)
  *		extension_val: Extension used to access predefined room(STRING)
  *		context: Context used to access predefined room(STRING)
  *		announce_user_count: Custom room option to turn on/off announcing user count to entering users(BOOLEAN[true or false])
  *		music_on_hold_for_single_user: Custom room option to turn on/off music on hold when a single user is in a room(BOOLEAN[true or false])
  *		exit_room_via_pound: Custom room option to turn on/off the ability for users to exit the room via the # key(BOOLEAN[true or false])
  *		present_menu_via_star: Custom room option to turn on/off the ability for users to be presented a menu via the * key(BOOLEAN[true or false])
  *		announce_user_join_leave: Custom room option to turn on/off the ability to announce the name of incoming users.
  *		disable_join_leave_notification: Custom room option to turn on/off a chime notification of entering and exiting users.
  *		record: Custom room option to turn on/off recording of a room.
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given conference room with specified configuration
  *			Takes: the location you wish to add the conference room to, the tenant that you would like to add the conference room to
  *			Returns: true if successful false if not
  */
  class ISymphonyConferenceRoom {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $original_name = "";
 	
 	//Property attributes
 	var $name = "";
 	var $predefined = "";
 	var $room_number = "";
 	var $extension_val = "";
 	var $context = "";
 	var $announce_user_count = "";
 	var $music_on_hold_for_single_user = "";
 	var $exit_room_via_pound = "";
 	var $present_menu_via_star = "";
 	var $announce_user_join_leave = "";
 	var $disable_join_leave_notification = "";
 	var $record = "";
	
 	//Methods
 	function update() {
 		
 		global $conferenceRoomPropertyArray;
 		
 		//Move to conference room mode
		if(!moveToConferenceRoom($this->mode_location, $this->mode_tenant, $this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($conferenceRoomPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant) {
 		
 		global $conferenceRoomPropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new meetme {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($conferenceRoomPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			return true;
			
		} else {
			return false;
		}
 	}
 } 

 /*###############################################
  * Class that represents a permission group's properties
  * Attributes: 
  *		name: Name of permission group(STRING)
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission group with specified configuration
  *			Takes: the location you wish to add the permission group to, the tenant that you would like to add the permission group to
  *			Returns: true if successful false if not
  */
  class ISymphonyPermissionGroup {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $original_name = "";
 	
 	//Property attributes
 	var $name = "";
	
 	//Methods
 	function update() {
 		
 		global $permissionGroupPropertyArray;
 		
 		//Move to permission group mode
		if(!moveToPermissionGroup($this->mode_location, $this->mode_tenant, $this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($permissionGroupPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant) {
 		
 		global $permissionGroupPropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new permgroup {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($permissionGroupPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			return true;
			
		} else {
			return false;
		}
 	}
 }   
 
 /*###############################################
  * Class that represents a status's properties
  * Attributes: 
  *		name: Name of status(STRING)
  *		type: Type of status(TYPE[available,unavailable,out])
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given status with specified configuration
  *			Takes: the location you wish to add the status to, the tenant that you would like to add the status to
  *			Returns: true if successful false if not
  */ 
  class ISymphonyStatus {
 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $original_name = "";
 	
 	//Property attributes
 	var $name = "";
 	var $type = "";
	
 	//Methods
 	function update() {
 		
 		global $statusPropertyArray;
 		
 		//Move to status mode
		if(!moveToStatus($this->mode_location, $this->mode_tenant, $this->original_name)) {
			return false;
		}
		
		//Modify properties
		foreach($statusPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if edit was successful so user can call subsequent commits
			$this->original_name = $this->name;
			return true;
			
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant) {
 		
 		global $statusPropertyArray;
 		
 		//Move to tenant mode
		if(!moveToTenant($location, $tenant)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new status {$this->name}")) {
			return false;
		}
		
		//Modify properties
		foreach($statusPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->original_name = $this->name;
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			return true;
			
		} else {
			return false;
		}
 	}
 } 
 
 /*###############################################
  * Class that represents the default local permissions
  * Attributes: 
  * 	call_voice_mail: Allows users to call there own extension's voice mail(BOOLEAN[allow or deny])
  * 	hold_calls: Allows users to place their calls on hold(BOOLEAN[allow or deny])
  * 	transfer_call_to_voice_mail: Allows users to transfer call to their voice mail(BOOLEAN[allow or deny])
  * 	mute: Allows users to mute/unmute themselves when in a conference room or barged call(BOOLEAN[allow or deny])
  * 	record: Allows users to record their calls(BOOLEAN[allow or deny])
  * 	hangup: Allows users to hangup their calls via the panel(BOOLEAN[allow or deny])
  * 	set_user_status_note: Allows users to set their status notes and return time(BOOLEAN[allow or deny])	
  * 	call_cell_phone: Allows users to call their cell phone(BOOLEAN[allow or deny])
  * 	add_extension_directory: Allows users to add extension directories(BOOLEAN[allow or deny])	
  * 	set_user_status: Allows users to set their status(BOOLEAN[allow or deny])
  * 	transfer_call_to_cell_phone: Allows users to transfer calls to their cell phone(BOOLEAN[allow or deny])
  * 	agent_login: Allows users to login/logout of their agent via the panel(BOOLEAN[allow or deny])
  * 	add_temp_meetme_room: Allows users to create temporary conference rooms BOOLEAN[allow or deny]
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */ 
  class ISymphonyDefaultLocalPermission {
 	 	
 	//Property attributes
 	var $call_voice_mail = "";
 	var $hold_calls = "";
 	var $transfer_call_to_voice_mail = "";
 	var $mute = "";
 	var $record = "";
 	var $hangup = "";
 	var $set_user_status_note = "";
 	var $call_cell_phone = "";
 	var $add_extension_directory = "";
 	var $set_user_status = "";
 	var $transfer_call_to_cell_phone = "";
 	var $agent_login = "";
 	var $add_temp_meetme_room = "";
	
 	//Methods
 	function update() {
 		
 		global $localPermissionPropertyArray;
 		
 		//Move to default local permission mode
		if(!moveToDefaultLocalPermission()) {
			return false;
		}
				
		//Modify properties
		foreach($localPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }    
 
 /*###############################################
  * Class that represents the default remote permissions
  * Attributes: 
  *		call_voice_mail: Allows user to call remote extension voice mail(BOOLEAN[allow or deny])
  *		transfer_to: Allows users to transfer calls to remote extensions(BOOLEAN[allow or deny])
  *		transfer_call_to_voice_mail: Allows users to transfer calls to remote extension voice mail(BOOLEAN[allow or deny])
  *		steal_call: Allows users to steal calls from remote extensions(BOOLEAN[allow or deny])
  *		record: Allows users to record remote extension calls(BOOLEAN[allow or deny])	
  *		originate_to: Allows users to originate calls to remote extensions via the panel(BOOLEAN[allow or deny])
  *		email: Allows users to email remote extensions(BOOLEAN[allow or deny])	
  *		call_cell_phone: Allows users to call remote extension cell phones(BOOLEAN[allow or deny])
  *		barge: Allows users to barge in on remote extension calls(BOOLEAN[allow or deny])
  *		transfer_call_to_cell_phone: Allows users to transfer calls to remote extension voice mail(BOOLEAN[allow or deny])
  *		chat: Allows users to initiate chat sessions with remote extensions(BOOLEAN[allow or deny])
  *		agent_login: Allows users to login/logout remote extension agents(BOOLEAN[allow or deny])	
  *		view_calls: Allows users to see remote extension call status(BOOLEAN[allow or deny])	
  *		view_caller_id: Allows users to see remote extension callerID(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */ 
  class ISymphonyDefaultRemotePermission {
 	 	
 	//Property attributes
	var $call_voice_mail = "";
	var $transfer_to = "";
	var $transfer_call_to_voice_mail = "";
	var $steal_call = "";
	var $record = "";
	var $originate_to = "";
	var $email = "";
	var $call_cell_phone = "";
	var $barge = "";
	var $transfer_call_to_cell_phone = "";
	var $chat = "";
	var $agent_login = "";	
	var $view_calls = "";	
	var $view_caller_id = "";
	
 	//Methods
 	function update() {
 		
 		global $remotePermissionPropertyArray;
 		
 		//Move to default remote permission mode
		if(!moveToDefaultRemotePermission()) {
			return false;
		}
				
		//Modify properties
		foreach($remotePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }  
 
 /*###############################################
  * Class that represents the default park permissions
  * Attributes: 
  *		park_call: Allows users to park calls(BOOLEAN[allow or deny])
  *		set_parked_call_note: Allows users to set parked call notes(BOOLEAN[allow or deny])
  *		unpark_call: Allows users to take calls from the parking lot via the panel(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */
  class ISymphonyDefaultParkPermission {
 	 	
 	//Property attributes
	var $park_call = "";
	var $set_parked_call_note = "";
	var $unpark_call = "";
	
 	//Methods
 	function update() {
 		
 		global $parkPermissionPropertyArray;
 		
 		//Move to default park permission mode
		if(!moveToDefaultParkPermission()) {
			return false;
		}
				
		//Modify properties
		foreach($parkPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }    
 
 /*###############################################
  * Class that represents the default queue permissions
  * Attributes: 
  *		transfer_to: Allows users to transfer calls to a queue(BOOLEAN[allow or deny])
  *		steal_call: Allows users to steal calls from a queue(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */ 
  class ISymphonyDefaultQueuePermission {
 	 	
 	//Property attributes
	var $transfer_to = "";
	var $steal_call = "";

 	//Methods
 	function update() {
 		
 		global $queuePermissionPropertyArray;
 		
 		//Move to default queue permission mode
		if(!moveToDefaultQueuePermission()) {
			return false;
		}
				
		//Modify properties
		foreach($queuePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }  
 
 /*###############################################
  * Class that represents the default conference room permissions
  * Attributes: 
  *		steal_call: Allows users to steal calls from conference rooms(BOOLEAN[allow or deny])
  *		transfer_to: Allows users to transfer calls to conference rooms(BOOLEAN[allow or deny])
  *		originate_to: Allows users to originate calls to a conference room(BOOLEAN[allow or deny])
  *		mute_users: Allows users to mute conference room users(BOOLEAN[allow or deny])
  *		kick_users: Allows users to kick conference room users from the room(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */  
  class ISymphonyDefaultConferenceRoomPermission {
 	 	
 	//Property attributes
	var $steal_call = "";
	var $transfer_to = "";
	var $originate_to = "";
	var $mute_users = "";
	var $kick_users = "";
 	
 	//Methods
 	function update() {
 		
 		global $conferenceRoomPermissionPropertyArray;
 		
 		//Move to default conference room permission mode
		if(!moveToDefaultConferenceRoomPermission()) {
			return false;
		}
				
		//Modify properties
		foreach($conferenceRoomPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }    
 
 /*###############################################
  * Class that represents a group's local permissions
  * Attributes: 
  * 	call_voice_mail: Allows users to call there own extension's voice mail(BOOLEAN[allow or deny])
  * 	hold_calls: Allows users to place their calls on hold(BOOLEAN[allow or deny])
  * 	transfer_call_to_voice_mail: Allows users to transfer call to their voice mail(BOOLEAN[allow or deny])
  * 	mute: Allows users to mute/unmute themselves when in a conference room or barged call(BOOLEAN[allow or deny])
  * 	record: Allows users to record their calls(BOOLEAN[allow or deny])
  * 	hangup: Allows users to hangup their calls via the panel(BOOLEAN[allow or deny])
  * 	set_user_status_note: Allows users to set their status notes and return time(BOOLEAN[allow or deny])	
  * 	call_cell_phone: Allows users to call their cell phone(BOOLEAN[allow or deny])
  * 	add_extension_directory: Allows users to add extension directories(BOOLEAN[allow or deny])	
  * 	set_user_status: Allows users to set their status(BOOLEAN[allow or deny])
  * 	transfer_call_to_cell_phone: Allows users to transfer calls to their cell phone(BOOLEAN[allow or deny])
  * 	agent_login: Allows users to login/logout of their agent via the panel(BOOLEAN[allow or deny])
  * 	add_temp_meetme_room: Allows users to create temporary conference rooms BOOLEAN[allow or deny]
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */ 
  class ISymphonyGroupLocalPermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_group = "";	
 	 	
 	//Property attributes
 	var $call_voice_mail = "";
 	var $hold_calls = "";
 	var $transfer_call_to_voice_mail = "";
 	var $mute = "";
 	var $record = "";
 	var $hangup = "";
 	var $set_user_status_note = "";
 	var $call_cell_phone = "";
 	var $add_extension_directory = "";
 	var $set_user_status = "";
 	var $transfer_call_to_cell_phone = "";
 	var $agent_login = "";
 	var $add_temp_meetme_room = "";
	
 	//Methods
 	function update() {
 		
 		global $localPermissionPropertyArray;
 		
 		//Move to group local permission mode
		if(!moveToLocalGroupPermission($this->mode_location, $this->mode_tenant, $this->mode_group)) {
			return false;
		}
				
		//Modify properties
		foreach($localPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }  
 
  /*###############################################
  * Class that represents a group's park permissions
  * Attributes: 
  *		park_call: Allows users to park calls(BOOLEAN[allow or deny])
  *		set_parked_call_note: Allows users to set parked call notes(BOOLEAN[allow or deny])
  *		unpark_call: Allows users to take calls from the parking lot via the panel(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */
  class ISymphonyGroupParkPermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_group = "";	 	
 	 	
 	//Property attributes
	var $park_call = "";
	var $set_parked_call_note = "";
	var $unpark_call = "";
	
 	//Methods
 	function update() {
 		
 		global $parkPermissionPropertyArray;
 		
 		//Move to group park permission mode
		if(!moveToParkGroupPermission($this->mode_location, $this->mode_tenant, $this->mode_group)) {
			return false;
		}
				
		//Modify properties
		foreach($parkPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }   
 
 /*###############################################
  * Class that represents a specified group remote permission set
  * Attributes: 
  *		call_voice_mail: Allows user to call remote extension voice mail(BOOLEAN[allow or deny])
  *		transfer_to: Allows users to transfer calls to remote extensions(BOOLEAN[allow or deny])
  *		transfer_call_to_voice_mail: Allows users to transfer calls to remote extension voice mail(BOOLEAN[allow or deny])
  *		steal_call: Allows users to steal calls from remote extensions(BOOLEAN[allow or deny])
  *		record: Allows users to record remote extension calls(BOOLEAN[allow or deny])	
  *		originate_to: Allows users to originate calls to remote extensions via the panel(BOOLEAN[allow or deny])
  *		email: Allows users to email remote extensions(BOOLEAN[allow or deny])	
  *		call_cell_phone: Allows users to call remote extension cell phones(BOOLEAN[allow or deny])
  *		barge: Allows users to barge in on remote extension calls(BOOLEAN[allow or deny])
  *		transfer_call_to_cell_phone: Allows users to transfer calls to remote extension voice mail(BOOLEAN[allow or deny])
  *		chat: Allows users to initiate chat sessions with remote extensions(BOOLEAN[allow or deny])
  *		agent_login: Allows users to login/logout remote extension agents(BOOLEAN[allow or deny])	
  *		view_calls: Allows users to see remote extension call status(BOOLEAN[allow or deny])	
  *		view_caller_id: Allows users to see remote extension callerID(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission with specified configuration
  *			Takes: location, tenant, permission group, extension
  *			Returns: true if successful false if not
  */  
  class ISymphonyGroupRemotePermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_group = "";
 	var $mode_extension = "";	 	
 	 	
 	//Property attributes
	var $call_voice_mail = "";
	var $transfer_to = "";
	var $transfer_call_to_voice_mail = "";
	var $steal_call = "";
	var $record = "";
	var $originate_to = "";
	var $email = "";
	var $call_cell_phone = "";
	var $barge = "";
	var $transfer_call_to_cell_phone = "";
	var $chat = "";
	var $agent_login = "";	
	var $view_calls = "";	
	var $view_caller_id = "";
	
 	//Methods
 	function update() {
 		
 		global $remotePermissionPropertyArray;
 		
 		//Move to group remote permission mode
		if(!moveToRemoteGroupPermission($this->mode_location, $this->mode_tenant, $this->mode_group, $this->mode_extension)) {
			return false;
		}
				
		//Modify properties
		foreach($remotePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant, $group, $extension) {
 		
 		global $remotePermissionPropertyArray;
 		
 		//Move to group mode
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new remote $extension")) {
			return false;
		}
		
		//Modify properties
		foreach($remotePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			$this->mode_group = $group;
			$this->mode_extension = $extension;
			return true;
			
		} else {
			return false;
		}
 	} 	
 }    
 
 /*###############################################
  * Class that represents a specified group queue permission set
  * Attributes: 
  *		transfer_to: Allows users to transfer calls to a queue(BOOLEAN[allow or deny])
  *		steal_call: Allows users to steal calls from a queue(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission with specified configuration
  *			Takes: location, tenant, permission group, queue
  *			Returns: true if successful false if not
  */   
  class ISymphonyGroupQueuePermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_group = "";
 	var $mode_queue = "";	 	
 	 	
 	//Property attributes
	var $transfer_to = "";
	var $steal_call = "";
	
 	//Methods
 	function update() {
 		
 		global $queuePermissionPropertyArray;
 		
 		//Move to group mode
		if(!moveToQueueGroupPermission($this->mode_location, $this->mode_tenant, $this->mode_group, $this->mode_queue)) {
			return false;
		}
				
		//Modify properties
		foreach($queuePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant, $group, $queue) {
 		
 		global $queuePermissionPropertyArray;
 		
 		//Move to group mode
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new queue $queue")) {
			return false;
		}
		
		//Modify properties
		foreach($queuePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			$this->mode_group = $group;
			$this->mode_queue = $queue;
			return true;
			
		} else {
			return false;
		}
 	} 	
 }  
 
 /*###############################################
  * Class that represents a specified group conference room permission set
  * Attributes: 
  *		steal_call: Allows users to steal calls from conference rooms(BOOLEAN[allow or deny])
  *		transfer_to: Allows users to transfer calls to conference rooms(BOOLEAN[allow or deny])
  *		originate_to: Allows users to originate calls to a conference room(BOOLEAN[allow or deny])
  *		mute_users: Allows users to mute conference room users(BOOLEAN[allow or deny])
  *		kick_users: Allows users to kick conference room users from the room(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission with specified configuration
  *			Takes: location, tenant, permission group, room
  *			Returns: true if successful false if not
  */  
  class ISymphonyGroupConferenceRoomPermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_group = "";
 	var $mode_room = "";	 	
 	 	
 	//Property attributes
	var $steal_call = "";
	var $transfer_to = "";
	var $originate_to = "";
	var $mute_users = "";
	var $kick_users = "";
	
 	//Methods
 	function update() {
 		
 		global $conferenceRoomPermissionPropertyArray;
 		
 		//Move to group mode
		if(!moveToConferenceRoomGroupPermission($this->mode_location, $this->mode_tenant, $this->mode_group, $this->mode_room)) {
			return false;
		}
				
		//Modify properties
		foreach($conferenceRoomPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant, $group, $room) {
 		
 		global $conferenceRoomPermissionPropertyArray;
 		
 		//Move to group mode
		if(!moveToPermissionGroup($location, $tenant, $group)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new meetme $room")) {
			return false;
		}
		
		//Modify properties
		foreach($conferenceRoomPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			$this->mode_group = $group;
			$this->mode_room = $room;
			return true;
			
		} else {
			return false;
		}
 	} 	
 }   
 
 /*###############################################
  * Class that represents a profile's local override permissions
  * Attributes: 
  * 	call_voice_mail: Allows users to call there own extension's voice mail(BOOLEAN[allow or deny])
  * 	hold_calls: Allows users to place their calls on hold(BOOLEAN[allow or deny])
  * 	transfer_call_to_voice_mail: Allows users to transfer call to their voice mail(BOOLEAN[allow or deny])
  * 	mute: Allows users to mute/unmute themselves when in a conference room or barged call(BOOLEAN[allow or deny])
  * 	record: Allows users to record their calls(BOOLEAN[allow or deny])
  * 	hangup: Allows users to hangup their calls via the panel(BOOLEAN[allow or deny])
  * 	set_user_status_note: Allows users to set their status notes and return time(BOOLEAN[allow or deny])	
  * 	call_cell_phone: Allows users to call their cell phone(BOOLEAN[allow or deny])
  * 	add_extension_directory: Allows users to add extension directories(BOOLEAN[allow or deny])	
  * 	set_user_status: Allows users to set their status(BOOLEAN[allow or deny])
  * 	transfer_call_to_cell_phone: Allows users to transfer calls to their cell phone(BOOLEAN[allow or deny])
  * 	agent_login: Allows users to login/logout of their agent via the panel(BOOLEAN[allow or deny])
  * 	add_temp_meetme_room: Allows users to create temporary conference rooms BOOLEAN[allow or deny]
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */ 
  class ISymphonyOverrideLocalPermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_profile = "";	
 	 	
 	//Property attributes
 	var $call_voice_mail = "";
 	var $hold_calls = "";
 	var $transfer_call_to_voice_mail = "";
 	var $mute = "";
 	var $record = "";
 	var $hangup = "";
 	var $set_user_status_note = "";
 	var $call_cell_phone = "";
 	var $add_extension_directory = "";
 	var $set_user_status = "";
 	var $transfer_call_to_cell_phone = "";
 	var $agent_login = "";
 	var $add_temp_meetme_room = "";
	
 	//Methods
 	function update() {
 		
 		global $localPermissionPropertyArray;
 		
 		//Move to profile local permission mode
		if(!moveToLocalOverridePermission($this->mode_location, $this->mode_tenant, $this->mode_profile)) {
			return false;
		}
				
		//Modify properties
		foreach($localPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }   
 
  /*###############################################
  * Class that represents a profile's park override permissions
  * Attributes: 
  *		park_call: Allows users to park calls(BOOLEAN[allow or deny])
  *		set_parked_call_note: Allows users to set parked call notes(BOOLEAN[allow or deny])
  *		unpark_call: Allows users to take calls from the parking lot via the panel(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  */
  class ISymphonyOverrideParkPermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_profilegroup = "";	 	
 	 	
 	//Property attributes
	var $park_call = "";
	var $set_parked_call_note = "";
	var $unpark_call = "";
	
 	//Methods
 	function update() {
 		
 		global $parkPermissionPropertyArray;
 		
 		//Move to profile park permission mode
		if(!moveToParkOverridePermission($this->mode_location, $this->mode_tenant, $this->mode_profile)) {
			return false;
		}
				
		//Modify properties
		foreach($parkPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 }   

 /*###############################################
  * Class that represents a profile's specified remote override permission set
  * Attributes: 
  *		call_voice_mail: Allows user to call remote extension voice mail(BOOLEAN[allow or deny])
  *		transfer_to: Allows users to transfer calls to remote extensions(BOOLEAN[allow or deny])
  *		transfer_call_to_voice_mail: Allows users to transfer calls to remote extension voice mail(BOOLEAN[allow or deny])
  *		steal_call: Allows users to steal calls from remote extensions(BOOLEAN[allow or deny])
  *		record: Allows users to record remote extension calls(BOOLEAN[allow or deny])	
  *		originate_to: Allows users to originate calls to remote extensions via the panel(BOOLEAN[allow or deny])
  *		email: Allows users to email remote extensions(BOOLEAN[allow or deny])	
  *		call_cell_phone: Allows users to call remote extension cell phones(BOOLEAN[allow or deny])
  *		barge: Allows users to barge in on remote extension calls(BOOLEAN[allow or deny])
  *		transfer_call_to_cell_phone: Allows users to transfer calls to remote extension voice mail(BOOLEAN[allow or deny])
  *		chat: Allows users to initiate chat sessions with remote extensions(BOOLEAN[allow or deny])
  *		agent_login: Allows users to login/logout remote extension agents(BOOLEAN[allow or deny])	
  *		view_calls: Allows users to see remote extension call status(BOOLEAN[allow or deny])	
  *		view_caller_id: Allows users to see remote extension callerID(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission with specified configuration
  *			Takes: location, tenant, profile, extension
  *			Returns: true if successful false if not
  */  
  class ISymphonyOverrideRemotePermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_proile = "";
 	var $mode_extension = "";	 	
 	 	
 	//Property attributes
	var $call_voice_mail = "";
	var $transfer_to = "";
	var $transfer_call_to_voice_mail = "";
	var $steal_call = "";
	var $record = "";
	var $originate_to = "";
	var $email = "";
	var $call_cell_phone = "";
	var $barge = "";
	var $transfer_call_to_cell_phone = "";
	var $chat = "";
	var $agent_login = "";	
	var $view_calls = "";	
	var $view_caller_id = "";
	
 	//Methods
 	function update() {
 		
 		global $remotePermissionPropertyArray;
 		
 		//Move to profile remote permission mode
		if(!moveToRemoteOverridePermission($this->mode_location, $this->mode_tenant, $this->mode_profile, $this->mode_extension)) {
			return false;
		}
				
		//Modify properties
		foreach($remotePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant, $profile, $extension) {
 		
 		global $remotePermissionPropertyArray;
 		
 		//Move to profile mode
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new remote $extension")) {
			return false;
		}
		
		//Modify properties
		foreach($remotePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			$this->mode_profile = $profile;
			$this->mode_extension = $extension;
			return true;
			
		} else {
			return false;
		}
 	} 	
 }    
 
 /*###############################################
  * Class that represents a profile's specified override queue permission set
  * Attributes: 
  *		transfer_to: Allows users to transfer calls to a queue(BOOLEAN[allow or deny])
  *		steal_call: Allows users to steal calls from a queue(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission with specified configuration
  *			Takes: location, tenant, profile, queue
  *			Returns: true if successful false if not
  */   
  class ISymphonyOverrideQueuePermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_profile = "";
 	var $mode_queue = "";	 	
 	 	
 	//Property attributes
	var $transfer_to = "";
	var $steal_call = "";
	
 	//Methods
 	function update() {
 		
 		global $queuePermissionPropertyArray;
 		
 		//Move to profile queue permission mode
		if(!moveToQueueOverridePermission($this->mode_location, $this->mode_tenant, $this->mode_profile, $this->mode_queue)) {
			return false;
		}
				
		//Modify properties
		foreach($queuePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant, $profile, $queue) {
 		
 		global $queuePermissionPropertyArray;
 		
 		//Move to profile mode
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new queue $queue")) {
			return false;
		}
		
		//Modify properties
		foreach($queuePermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			$this->mode_profile = $profile;
			$this->mode_queue = $queue;
			return true;
			
		} else {
			return false;
		}
 	} 	
 }  
 
 /*###############################################
  * Class that represents a profile's specified override conference room permission set
  * Attributes: 
  *		steal_call: Allows users to steal calls from conference rooms(BOOLEAN[allow or deny])
  *		transfer_to: Allows users to transfer calls to conference rooms(BOOLEAN[allow or deny])
  *		originate_to: Allows users to originate calls to a conference room(BOOLEAN[allow or deny])
  *		mute_users: Allows users to mute conference room users(BOOLEAN[allow or deny])
  *		kick_users: Allows users to kick conference room users from the room(BOOLEAN[allow or deny])
  *
  * Methods:
  *		update: Commits changes made to the property configuration
  *			Takes: nothing
  *			Returns: true if successful false if not
  *		add: Adds the given permission with specified configuration
  *			Takes: location, tenant, profile, room
  *			Returns: true if successful false if not
  */  
  class ISymphonyOverrideConferenceRoomPermission {
 	 	
 	//Mode attributes
 	var $mode_location = "";
 	var $mode_tenant = "";
 	var $mode_profile = "";
 	var $mode_room = "";	 	
 	 	
 	//Property attributes
	var $steal_call = "";
	var $transfer_to = "";
	var $originate_to = "";
	var $mute_users = "";
	var $kick_users = "";
	
 	//Methods
 	function update() {
 		
 		global $conferenceRoomPermissionPropertyArray;
 		
 		//Move to profile mode
		if(!moveToConferenceRoomOverridePermission($this->mode_location, $this->mode_tenant, $this->mode_profile, $this->mode_room)) {
			return false;
		}
				
		//Modify properties
		foreach($conferenceRoomPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit changes
		if(checkAndSetErrorNone("commit")) {
			return true;
		} else {
			return false;
		}
 	}
 	
 	function add($location, $tenant, $profile, $room) {
 		
 		global $conferenceRoomPermissionPropertyArray;
 		
 		//Move to profile mode
		if(!moveToProfile($location, $tenant, $profile)) {
			return false;
		}
		
		//Initiate new command
		if(!checkAndSetErrorNone("new meetme $room")) {
			return false;
		}
		
		//Modify properties
		foreach($conferenceRoomPermissionPropertyArray as $val) {
			if(!checkAndSetErrorNone("$val {$this->$val}")) {
				return false;
			}
		}
		
		//Commit addition
		if(checkAndSetErrorNone("commit")) {
			
			//Set mode properties if addition was successful so user can call commit
			$this->mode_location = $location;
			$this->mode_tenant = $tenant;
			$this->mode_profile = $profile;
			$this->mode_room = $room;
			return true;
			
		} else {
			return false;
		}
 	} 	
 }   
?>