<?php
/*
 *Name         : helper.php
 *Author       : Michael Yara
 *Created      : June 15, 2008
 *Last Updated : June 18, 2008
 *History      : 0.1 Beta  
 *Purpose      : Contains utility functions used by the library. 
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

/*###############################################
 * Sends a message to the server and listens for a reply
 * Takes: request
 * Returns: an error or the server reply
 */
function sendAndRecive($request) {
	
	global $SOCKET, $READ_LENGTH, $NEW_LINE;
	
	//Add new line to request
	$request = $request . $NEW_LINE;
		
	//Check if socket is initialized 
	if(($SOCKET !== null) && ($SOCKET !== false)) {
		
		//Read out prompt
		if(($buf = socket_read($SOCKET, $READ_LENGTH, PHP_BINARY_READ)) !== false) {
			
			//Send request
			if(socket_write($SOCKET, $request, strlen($request))) {
				
				//Read reply					
				if(($buf = socket_read($SOCKET, $READ_LENGTH, PHP_BINARY_READ)) !== false) {
					return $buf;
				} else {
					return "Error: Could not read from socket. " . socket_strerror(socket_last_error());
				}
			} else {
				return "Error: Could not write to socket. " . socket_strerror(socket_last_error());
			}
		} else {
			return "Error: Could not read from socket. " . socket_strerror(socket_last_error());
		}
	} else {
		return "Error: Socket not connected. Call iSymphonyConnect() to initialize connection.";
	}	
}
	
/*###############################################
 * Checks if reply was an error
 * Takes: server reply
 * Returns: true or false
 */	
function isError($reply) {
	return strpos($reply, "Error") === 0;
}	

/*###############################################
 * Makes a request that returns nothing and checks reply for error. If error is found $ISERROR is set
 * Takes: request
 * Returns: true if successful and false if error 
 */
function checkAndSetErrorNone($request) {
	
	global $ISERROR;
	
	if(isError($reply = sendAndRecive($request))) {
		$ISERROR = $reply;
		return false;
	} else {
		return true;
	}
}

/*###############################################
 * Makes a request that returns a string and checks reply for error. If error is found $ISERROR is set
 * Takes: request
 * Returns: the string if successful and false if error 
 */
function checkAndSetErrorString($request) {
	
	global $ISERROR;
	
	if(isError($reply = sendAndRecive($request))) {
		$ISERROR = $reply;
		return false;
	} else {
		return trim($reply);
	}
}

/*###############################################
 * Makes a request that returns a list and checks reply for error. If error is found $ISERROR is set
 * Takes: request
 * Returns: an array of values if successful and false if error 
 */
function checkAndSetErrorList($request) {
	
	global $ISERROR, $NEW_LINE;
	
	if(isError($reply = sendAndRecive($request))) {
		$ISERROR = $reply;
		return false;
	} else {
		$explodeArray = explode($NEW_LINE, trim($reply));
		$returnArray = array();
	
		//Search array for empty values and remove them
		foreach($explodeArray as $val) {
			if(strlen($val) != 0) {
				array_push($returnArray, $val);
			}
		}
				
		return $returnArray;
	}
}


/*###############################################
 * Coverts a property list into a associative key => value array 
 * Takes: an array of properties with values structured like so (property = value)
 * Returns: an associative array with the structure (property => value)
 */
function convertPropertyArray($array) {
	
	$returnArray = array();
	
	foreach($array as $val) {
	
		//Check if the property has a value and set the key value pair appropriately 
		if(endsWith($val," =")) {
			$keyValuePair = array(rtrim($val," =") ,"");
		} else if(endsWith($val," = ")) {
			$keyValuePair = array(rtrim($val," = "),"");
		} else {
			$keyValuePair = explode(" = ", $val);
		}

		$returnArray[$keyValuePair[0]] = $keyValuePair[1];
	}
	
	return $returnArray;
}

/*###############################################
 * Checks if a string ends with another specified string
 * Takes: a string, the string you wish to check is at the end
 * Returns: true if $endString is at the end of $string false if not
 */
function endsWith($string, $endString){
    return strrpos($string, $endString) === strlen($string)-strlen($endString);
}


?>