<?php
/*
 *Name         : navigation.php
 *Author       : Michael Yara
 *Created      : June 15, 2008
 *Last Updated : June 18, 2008
 *History      : 0.1 Beta  
 *Purpose      : Contains functions used to navigate through different modes of the CLI. 
 *Copyright    : 2008 HEHE Enterprises, LLC
 */

/*###############################################
 * Moves CLI to server mode
 * Takes: nothing
 * Returns: true if successful or false if not
 */
function moveToServer() {

	//Move to server
	return checkAndSetErrorNone("server");
}

/*###############################################
 * Moves CLI to location mode
 * Takes: location name
 * Returns: true if successful or false if not
 */
function moveToLocation($location) {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}	
		
	//Move to location
	return checkAndSetErrorNone("location $location");
}

/*###############################################
 * Moves CLI to tenant mode
 * Takes: location name, tenant name
 * Returns: true if successful or false if not
 */
function moveToTenant($location, $tenant) {
	
	//Move to location
	if(!moveToLocation($location)) {
		return false;
	}
	
	//Move to tenant
	return checkAndSetErrorNone("tenant $tenant");
}

/*###############################################
 * Moves CLI to extension mode
 * Takes: location name, tenant name, extension name
 * Returns: true if successful or an error if not
 */
function moveToExtension($location, $tenant, $extension) {
	
	//Move to tenant
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Move to extension
	return checkAndSetErrorNone("extension $extension");
}

/*###############################################
 * Moves CLI to profile mode
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or an error if not
 */
function moveToProfile($location, $tenant, $profile) {
	
	//Move to tenant
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Move to profile
	return checkAndSetErrorNone("profile $profile");
}

/*###############################################
 * Moves CLI to queue mode
 * Takes: location name, tenant name, queue name
 * Returns: true if successful or an error if not
 */
function moveToQueue($location, $tenant, $queue) {
	
	//Move to tenant
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Move to queue
	return checkAndSetErrorNone("queue $queue");
}

/*###############################################
 * Moves CLI to conference room mode
 * Takes: location name, tenant name, room name
 * Returns: true if successful or an error if not
 */
function moveToConferenceRoom($location, $tenant, $room) {
	
	//Move to tenant
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Move to conference room
	return checkAndSetErrorNone("meetme $room");
}

/*###############################################
 * Moves CLI to status mode
 * Takes: location name, tenant name, status name
 * Returns: true if successful or an error if not
 */
function moveToStatus($location, $tenant, $status) {
	
	//Move to tenant
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Move to status
	return checkAndSetErrorNone("status $status");
}

/*###############################################
 * Moves CLI to permission group mode
 * Takes: location name, tenant name, group name
 * Returns: true if successful or an error if not
 */
function moveToPermissionGroup($location, $tenant, $group) {
	
	//Move to tenant
	if(!moveToTenant($location, $tenant)) {
		return false;
	}
	
	//Move to permission group
	return checkAndSetErrorNone("permgroup $group");
}

/*###############################################
 * Moves CLI to default local permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultLocalPermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default local permissions
	return checkAndSetErrorNone("permission local");
}

/*###############################################
 * Moves CLI to default remote permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultRemotePermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default remote permissions
	return checkAndSetErrorNone("permission remote");
}

/*###############################################
 * Moves CLI to default queue permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultQueuePermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default queue permissions
	return checkAndSetErrorNone("permission queue");
}

/*###############################################
 * Moves CLI to default park permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultParkPermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default park permissions
	return checkAndSetErrorNone("permission park");
}

/*###############################################
 * Moves CLI to default conference room permission mode
 * Takes: nothing
 * Returns: true if successful or an error if not
 */
function moveToDefaultConferenceRoomPermission() {
	
	//Move to server
	if(!moveToServer()) {
		return false;
	}
	
	//Move to default conference room permissions
	return checkAndSetErrorNone("permission meetme");
}

/*###############################################
 * Moves CLI to local permission override mode
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or an error if not
 */
function moveToLocalOverridePermission($location, $tenant, $profile) {
	
	//Move to profile
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Move to local permission override
	return checkAndSetErrorNone("permission local");
}

/*###############################################
 * Moves CLI to park permission override mode
 * Takes: location name, tenant name, profile name
 * Returns: true if successful or an error if not
 */
function moveToParkOverridePermission($location, $tenant, $profile) {
	
	//Move to profile
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Move to park permission override
	return checkAndSetErrorNone("permission park");
}

/*###############################################
 * Moves CLI to remote permission override mode
 * Takes: location name, tenant name, profile name, extension name
 * Returns: true if successful or an error if not
 */
function moveToRemoteOverridePermission($location, $tenant, $profile, $extension) {
	
	//Move to profile
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Move to remote permission override
	return checkAndSetErrorNone("permission remote $extension");
}

/*###############################################
 * Moves CLI to queue permission override mode
 * Takes: location name, tenant name, profile name, queue name
 * Returns: true if successful or an error if not
 */
function moveToQueueOverridePermission($location, $tenant, $profile, $queue) {
	
	//Move to profile
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Move to queue permission override
	return checkAndSetErrorNone("permission queue $queue");
}

/*###############################################
 * Moves CLI to conference room permission override mode
 * Takes: location name, tenant name, profile name, queue name
 * Returns: true if successful or an error if not
 */
function moveToConferenceRoomOverridePermission($location, $tenant, $profile, $room) {
	
	//Move to profile
	if(!moveToProfile($location, $tenant, $profile)) {
		return false;
	}
	
	//Move to conference room permission override
	return checkAndSetErrorNone("permission meetme $room");
}

/*###############################################
 * Moves CLI to local group permission mode
 * Takes: location name, tenant name, permission group name
 * Returns: true if successful or an error if not
 */
function moveToLocalGroupPermission($location, $tenant, $group) {
	
	//Move to permission group
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Move to group local permission
	return checkAndSetErrorNone("permission local");
}

/*###############################################
 * Moves CLI to park group permission mode
 * Takes: location name, tenant name, permission group name
 * Returns: true if successful or an error if not
 */
function moveToParkGroupPermission($location, $tenant, $group) {
	
	//Move to permission group
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Move to group park permission
	return checkAndSetErrorNone("permission park");
}

/*###############################################
 * Moves CLI to remote group permission mode
 * Takes: location name, tenant name, permission group name, extension name
 * Returns: true if successful or an error if not
 */
function moveToRemoteGroupPermission($location, $tenant, $group, $extension) {
	
	//Move to permission group
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Move to group remote permission
	return checkAndSetErrorNone("permission remote $extension");
}

/*###############################################
 * Moves CLI to queue group permission mode
 * Takes: location name, tenant name, permission group name, queue name
 * Returns: true if successful or an error if not
 */
function moveToQueueGroupPermission($location, $tenant, $group, $queue) {
	
	//Move to permission group
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Move to group queue permission
	return checkAndSetErrorNone("permission queue $queue");
}

/*###############################################
 * Moves CLI to conference room group permission mode
 * Takes: location name, tenant name, permission group name, conference room name
 * Returns: true if successful or an error if not
 */
function moveToConferenceRoomGroupPermission($location, $tenant, $group, $room) {
	
	//Move to permission group
	if(!moveToPermissionGroup($location, $tenant, $group)) {
		return false;
	}
	
	//Move to group conference room permission
	return checkAndSetErrorNone("permission meetme $room");
}
?>